use std::f32::consts::PI;

// this function is used by both the modulator and demodulator to generate chirps, so it's split off here 
// to ensure that it is always exactly the same for both modules.

pub fn generate
(
	freq_lo: f32, // 1.0 = Nyquist limit
	freq_hi: f32,
	length: f32, // samples
	tail: f32, // proportion
) ->
	Vec<f32>
{
	let freq_delta = (freq_hi - freq_lo) / (length * 2.0);
	
	let env_falloff = tail * length / 9.6;
	let env_bot = length * tail;
	let env_top = length * (1.0 - tail);

	let mut new_chirp:Vec<f32> = Vec::with_capacity(length.round() as usize);
	for i in 0..(length.round() as usize)
	{
		let t = i as f32;
		let env_center = t.max(env_bot).min(env_top);

		let env:f32 = 
		(
			(
				-0.5 * 
				(
					(t - env_center) / env_falloff
				)
				.powi(2)
			)
			.exp()
		);
		
		new_chirp.push(
			env *
			(
				PI * t * 
				(
					freq_lo 
					+ 
					(t * freq_delta)
				)
			)
			.sin()
		);
	}

	return new_chirp;
}

