#![allow(unused_parens)]

use crate::chirp;

pub struct Enmodulator
{
	symbol_len: usize,
	guard_pad: usize,
	symbol_rise: Vec<f32>,
	symbol_fall: Vec<f32>,
}

impl Enmodulator
{
	pub fn new
	(
		center_freq: f32,
		deviation: f32,
		symbol_len: f32,
	) ->
		Enmodulator
	{
		let lower_freq = (center_freq - deviation).max(0.0).min(1.0);
		let upper_freq = (center_freq + deviation).max(0.0).min(1.0);

		let symbol_rise:Vec<f32> = chirp::generate(lower_freq, upper_freq, symbol_len, 0.125);
		let symbol_fall:Vec<f32> = chirp::generate(upper_freq, lower_freq, symbol_len, 0.125);

		Enmodulator
		{
			symbol_len: symbol_len.round() as usize,
			guard_pad: (symbol_len / 8.0).round() as usize,
			symbol_rise: symbol_rise,
			symbol_fall: symbol_fall,
		}
	}

	pub fn process
	(
		&self,
		byte: u8,
	) ->
		Vec<f32>
	{
		let mut new_signal:Vec<f32> = 
		(
			Vec::with_capacity(
				(8 * self.symbol_len) 
				+ 
				(2 * self.guard_pad)
			)
		);
		
		new_signal.append(&mut vec![0.0; self.guard_pad]);

		for i in 0..8
		{
			new_signal.extend_from_slice(
				[
					&self.symbol_fall,
					&self.symbol_rise,
				] [
					(
						(
							byte 
							& 
							(0b1000_0000 >> i) 
						)
						!= 0
					) 
					as usize
				]
			);
		}
		
		new_signal.append(&mut vec![0.0; self.guard_pad]);

		return new_signal;
	}
}
