#![allow(dead_code)]

#[derive(Clone)]
pub struct Buffer<T>
	where T: Copy + Clone + Default
{ 
	data: Vec<T>,         // the actual data in the buffer; length never changes after construction
	front: usize,         // where in the data vec is the first element?
	bound: usize,         // len-1; modulo-length can be done very fast by just bitwise-anding with this, since the length is 2^n
	size: usize,          // len; separate from bound just to make the code look nicer
}

impl<T> Buffer<T>
	where T: Copy + Clone + Default
{

	pub fn new
	(
		size: usize,
	) ->
		Buffer<T>
	{
		if size == 0
		{
			panic!("attempt to create a posi::Buffer with zero size");
		}
		let true_size = size.next_power_of_two();
		Buffer {
			data: vec![Default::default(); true_size],
			front: 0,
			bound: true_size - 1,
			size: size,
		}
	}

	pub fn shift_in
	(
		&mut self,
		input: T,
	) {
		self.front = self.front.wrapping_sub(1);
		self.front &= self.bound;	
		unsafe 
		{
			*self.data.get_unchecked_mut(self.front) = input;
		}
	}

	pub fn get
	(
		&self,
		index: usize,
	) -> 
		T
	{
		unsafe 
		{
			*self.data.get_unchecked((self.front + index) & self.bound)
		}
	}

	pub fn first
	(
		&self,
	) -> 
		T
	{
		self.get(0)
	}

	pub fn mid
	(
		&self,
	) ->
		T
	{
		self.get(self.size / 2)
	}

	pub fn above_mid
	(
		&self,
		index: usize
	) ->
		T
	{
		self.get((self.size / 2).wrapping_add(index))
	}
 
	pub fn below_mid
	(
		&self,
		index: usize
	) ->
		T
	{
		self.get((self.size / 2).wrapping_sub(index))
	}
 
	pub fn last
	(
		&self,
	) ->
		T
	{
		self.get(self.size - 1)
	}

	pub fn len
	(
		&self,
	) -> 
		usize
	{
		self.size
	}

	pub fn to_vec
	(
		&self
	) ->
		Vec<T>
	{
		let mut output:Vec<T> = Vec::with_capacity(self.data.len());	
		let (upper_slice, lower_slice) = self.data.split_at(self.front);
		output.extend_from_slice(lower_slice);
		output.extend_from_slice(upper_slice);
		output.truncate(self.size);
		return output;
	}

}

